#!/usr/bin/env lua

local _G             = _G
local _VERSION       = _VERSION
local assert         = assert
local collectgarbage = collectgarbage
local dofile         = dofile
local error          = error
local getfenv        = getfenv
local getmetatable   = getmetatable
local ipairs         = ipairs
local load           = load
local loadfile       = loadfile
local loadstring     = loadstring
local module         = module
local next           = next
local pairs          = pairs
local pcall          = pcall
local print          = print
local rawequal       = rawequal
local rawget         = rawget
local rawset         = rawset
local require        = require
local select         = select
local setfenv        = setfenv
local setmetatable   = setmetatable
local tonumber       = tonumber
local tostring       = tostring
local type           = type
local unpack         = unpack
local xpcall         = xpcall

local coroutine = coroutine
local debug     = debug
local io        = io
local math      = math
local os        = os
local package   = package
local string    = string
local table     = table

require("multirand")
local multirand = multirand

module(...)

temp_dir = false  -- has to be set to a private directory (/tmp can be unsafe)

function escape(str)
  return (
    string.gsub(
      str,
      "[\001-\031\127\\#$&~_^%%{}]",
      function(char)
        local b = string.byte(char)
        if (b > 1 and b < 31) or b == 127 then
          return " "
        elseif
          char == "#" or char == "$" or char == "&" or char == "_" or
          char == "%" or char == "{" or char == "}"
        then
          return "\\" .. char
        else
          return "\\symbol{" .. b .. "}"
        end
      end
    )
  )
end

document_methods = {}

document_mt = {
  __index = document_methods,
  __call = function(...) return document_methods.write(...) end
}

function new_document()
  return setmetatable({}, document_mt)
end

function document_methods:write(...)
  local i = 1
  while true do
    local v = select(i, ...)
    if v == nil then
      break
    end
    self[#self+1] = v
    i = i + 1
  end
end

function document_methods:get_latex()
  local str = table.concat(self)
  for i in ipairs(self) do
    self[i] = nil
  end
  self[1] = str
  return str
end

function document_methods:get_pdf()
  -- TODO: proper escaping of shell commands (should not be a real risk)
  if not temp_dir then
    error("luatex.temp_dir not set")
  end
  local basename = temp_dir .. "/tmp.luatex_" .. multirand.string(16)
  local latex_file = assert(io.open(basename .. ".tex", "w"))
  latex_file:write(self:get_latex())
  latex_file:close()
  local result = os.execute(
    'latex -output-format=pdf "-output-directory=' .. temp_dir .. '" ' ..
    basename .. '< /dev/null > /dev/null 2> /dev/null'
  )
  if result ~= 0 then
    error('LaTeX failed, see "' .. basename .. '.log" for details.')
  end
  local pdf_file = assert(io.open(basename .. ".pdf", "r"))
  local pdf_data = pdf_file:read("*a")
  pdf_file:close()
  os.execute('rm -f "' .. basename .. '.*"')
  return pdf_data
end

--[[

require("luatex")
luatex.temp_dir = "."

local tex = luatex.new_document()

tex "\\documentclass[a4paper,12pt]{article}\n"
tex "\\usepackage{german}\n"
tex "\\usepackage{amsfonts}\n"
tex "\\usepackage{amssymb}\n"
tex "\\usepackage{ulem}\n"
tex "\\pagestyle{headings}\n"
tex "\\begin{document}\n"
tex "\\title{Demo}\n"
tex "\\author{John Doe}\n"
tex "\\date{\\small 25. August 2008}\n"
tex "\\maketitle\n"
tex "\\end{document}\n"

local pdf = tex:get_pdf()

--]]
